# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .email_mailgun_region_enum import EmailMailgunRegionEnum


class EmailProviderCredentialsSchemaApiKey(UniversalBaseModel):
    api_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    API Key
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    Domain
    """

    region: typing.Optional[EmailMailgunRegionEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Any

from .base import AuthenticationBase


class Social(AuthenticationBase):

    """Social provider's endpoints.

    Args:
        domain (str): Your auth0 domain (e.g: my-domain.us.auth0.com)
    """

    def login(self, access_token: str, connection: str, scope: str = "openid") -> Any:
        """Login using a social provider's access token

        Given the social provider's access_token and the connection specified,
        it will do the authentication on the provider and return a dict with
        the access_token and id_token. Currently, this endpoint only works for
        Facebook, Google, Twitter and Weibo.

        Args:
            access_token (str): social provider's access_token.

            connection (str): connection type (e.g: 'facebook')

        Returns:
            A dict with 'access_token' and 'id_token' keys.
        """

        return self.post(
            f"{self.protocol}://{self.domain}/oauth/access_token",
            data={
                "client_id": self.client_id,
                "access_token": access_token,
                "connection": connection,
                "scope": scope,
            },
        )



# This file was auto-generated by Fern from our API Definition.

ConnectionTenantIdAzureAd = str
"""
The Azure AD tenant ID as a UUID. The unique identifier for your Azure AD organization. Must be a valid 36-character UUID.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_auth_0_oauth_app import CreateFlowsVaultConnectionAuth0OauthApp
from .create_flows_vault_connection_auth_0_uninitialized import CreateFlowsVaultConnectionAuth0Uninitialized

CreateFlowsVaultConnectionAuth0 = typing.Union[
    CreateFlowsVaultConnectionAuth0OauthApp, CreateFlowsVaultConnectionAuth0Uninitialized
]



import unittest
from unittest import mock

from auth0.authentication.database import Database


class TestDatabase(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_signup(self, mock_post):
        d = Database("my.domain.com", "cid")

        # using only email and password
        d.signup(email="a@b.com", password="pswd", connection="conn")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/dbconnections/signup")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "email": "a@b.com",
                "password": "pswd",
                "connection": "conn",
            },
        )

        # Using also optional properties
        sample_meta = {"hobby": "surfing", "preference": {"color": "pink"}}
        d.signup(
            email="a@b.com",
            password="pswd",
            connection="conn",
            username="usr",
            user_metadata=sample_meta,
            given_name="john",
            family_name="doe",
            name="john doe",
            nickname="johnny",
            picture="avatars.com/john-doe",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/dbconnections/signup")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "email": "a@b.com",
                "password": "pswd",
                "connection": "conn",
                "username": "usr",
                "user_metadata": sample_meta,
                "given_name": "john",
                "family_name": "doe",
                "name": "john doe",
                "nickname": "johnny",
                "picture": "avatars.com/john-doe",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_change_password(self, mock_post):
        d = Database("my.domain.com", "cid")

        # ignores the password argument
        d.change_password(email="a@b.com", password="pswd", connection="conn")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/dbconnections/change_password")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "email": "a@b.com",
                "connection": "conn",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_change_password_with_organization_param(self, mock_post):
        d = Database("my.domain.com", "cid")

        # ignores the password argument
        d.change_password(
            email="a@b.com", password="pswd", connection="conn", organization="org_id"
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/dbconnections/change_password")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "email": "a@b.com",
                "connection": "conn",
                "organization": "org_id",
            },
        )



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventOrgDeletedObjectMetadata = typing.Dict[str, typing.Any]
"""
The metadata associated with the organization.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionUpdatedObject1OptionsTokenEndpointJwtcaAudFormatEnum = typing.Union[
    typing.Literal["issuer", "token_endpoint"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_pipedrive_oauth_code import CreateFlowsVaultConnectionPipedriveOauthCode
from .create_flows_vault_connection_pipedrive_token import CreateFlowsVaultConnectionPipedriveToken
from .create_flows_vault_connection_pipedrive_uninitialized import CreateFlowsVaultConnectionPipedriveUninitialized

CreateFlowsVaultConnectionPipedrive = typing.Union[
    CreateFlowsVaultConnectionPipedriveToken,
    CreateFlowsVaultConnectionPipedriveOauthCode,
    CreateFlowsVaultConnectionPipedriveUninitialized,
]



from .conftest import get_client, verify_request_count

from auth0.management import PermissionRequestPayload


def test_users_permissions_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "users.permissions.list_.0"
    client = get_client(test_id)
    client.users.permissions.list(
        id="id",
        per_page=1,
        page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id, "GET", "/users/id/permissions", {"per_page": "1", "page": "1", "include_totals": "true"}, 1
    )


def test_users_permissions_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "users.permissions.create.0"
    client = get_client(test_id)
    client.users.permissions.create(
        id="id",
        permissions=[
            PermissionRequestPayload(
                resource_server_identifier="resource_server_identifier",
                permission_name="permission_name",
            )
        ],
    )
    verify_request_count(test_id, "POST", "/users/id/permissions", None, 1)


def test_users_permissions_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "users.permissions.delete.0"
    client = get_client(test_id)
    client.users.permissions.delete(
        id="id",
        permissions=[
            PermissionRequestPayload(
                resource_server_identifier="resource_server_identifier",
                permission_name="permission_name",
            )
        ],
    )
    verify_request_count(test_id, "DELETE", "/users/id/permissions", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_form_response_content import CreateFormResponseContent
from ..types.form_ending_node import FormEndingNode
from ..types.form_ending_node_nullable import FormEndingNodeNullable
from ..types.form_languages import FormLanguages
from ..types.form_languages_nullable import FormLanguagesNullable
from ..types.form_messages import FormMessages
from ..types.form_messages_nullable import FormMessagesNullable
from ..types.form_node_list import FormNodeList
from ..types.form_node_list_nullable import FormNodeListNullable
from ..types.form_start_node import FormStartNode
from ..types.form_start_node_nullable import FormStartNodeNullable
from ..types.form_style import FormStyle
from ..types.form_style_nullable import FormStyleNullable
from ..types.form_summary import FormSummary
from ..types.form_translations import FormTranslations
from ..types.form_translations_nullable import FormTranslationsNullable
from ..types.forms_request_parameters_hydrate_enum import FormsRequestParametersHydrateEnum
from ..types.get_form_response_content import GetFormResponseContent
from ..types.list_forms_offset_paginated_response_content import ListFormsOffsetPaginatedResponseContent
from ..types.update_form_response_content import UpdateFormResponseContent
from .raw_client import AsyncRawFormsClient, RawFormsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FormsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFormsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFormsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFormsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]
            Forms successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.forms.list(
            page=1,
            per_page=1,
            include_totals=True,
            hydrate=["flow_count"],
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            hydrate=hydrate,
            request_options=request_options,
        )

    def create(
        self,
        *,
        name: str,
        messages: typing.Optional[FormMessages] = OMIT,
        languages: typing.Optional[FormLanguages] = OMIT,
        translations: typing.Optional[FormTranslations] = OMIT,
        nodes: typing.Optional[FormNodeList] = OMIT,
        start: typing.Optional[FormStartNode] = OMIT,
        ending: typing.Optional[FormEndingNode] = OMIT,
        style: typing.Optional[FormStyle] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFormResponseContent:
        """
        Parameters
        ----------
        name : str

        messages : typing.Optional[FormMessages]

        languages : typing.Optional[FormLanguages]

        translations : typing.Optional[FormTranslations]

        nodes : typing.Optional[FormNodeList]

        start : typing.Optional[FormStartNode]

        ending : typing.Optional[FormEndingNode]

        style : typing.Optional[FormStyle]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFormResponseContent
            Form successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.forms.create(
            name="name",
        )
        """
        _response = self._raw_client.create(
            name=name,
            messages=messages,
            languages=languages,
            translations=translations,
            nodes=nodes,
            start=start,
            ending=ending,
            style=style,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetFormResponseContent:
        """
        Parameters
        ----------
        id : str
            The ID of the form to retrieve.

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFormResponseContent
            Form successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.forms.get(
            id="id",
            hydrate=["flow_count"],
        )
        """
        _response = self._raw_client.get(id, hydrate=hydrate, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            The ID of the form to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.forms.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        messages: typing.Optional[FormMessagesNullable] = OMIT,
        languages: typing.Optional[FormLanguagesNullable] = OMIT,
        translations: typing.Optional[FormTranslationsNullable] = OMIT,
        nodes: typing.Optional[FormNodeListNullable] = OMIT,
        start: typing.Optional[FormStartNodeNullable] = OMIT,
        ending: typing.Optional[FormEndingNodeNullable] = OMIT,
        style: typing.Optional[FormStyleNullable] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateFormResponseContent:
        """
        Parameters
        ----------
        id : str
            The ID of the form to update.

        name : typing.Optional[str]

        messages : typing.Optional[FormMessagesNullable]

        languages : typing.Optional[FormLanguagesNullable]

        translations : typing.Optional[FormTranslationsNullable]

        nodes : typing.Optional[FormNodeListNullable]

        start : typing.Optional[FormStartNodeNullable]

        ending : typing.Optional[FormEndingNodeNullable]

        style : typing.Optional[FormStyleNullable]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateFormResponseContent
            Form successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.forms.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            messages=messages,
            languages=languages,
            translations=translations,
            nodes=nodes,
            start=start,
            ending=ending,
            style=style,
            request_options=request_options,
        )
        return _response.data


class AsyncFormsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFormsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFormsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFormsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]
            Forms successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.forms.list(
                page=1,
                per_page=1,
                include_totals=True,
                hydrate=["flow_count"],
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            hydrate=hydrate,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        name: str,
        messages: typing.Optional[FormMessages] = OMIT,
        languages: typing.Optional[FormLanguages] = OMIT,
        translations: typing.Optional[FormTranslations] = OMIT,
        nodes: typing.Optional[FormNodeList] = OMIT,
        start: typing.Optional[FormStartNode] = OMIT,
        ending: typing.Optional[FormEndingNode] = OMIT,
        style: typing.Optional[FormStyle] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFormResponseContent:
        """
        Parameters
        ----------
        name : str

        messages : typing.Optional[FormMessages]

        languages : typing.Optional[FormLanguages]

        translations : typing.Optional[FormTranslations]

        nodes : typing.Optional[FormNodeList]

        start : typing.Optional[FormStartNode]

        ending : typing.Optional[FormEndingNode]

        style : typing.Optional[FormStyle]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFormResponseContent
            Form successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.create(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            messages=messages,
            languages=languages,
            translations=translations,
            nodes=nodes,
            start=start,
            ending=ending,
            style=style,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetFormResponseContent:
        """
        Parameters
        ----------
        id : str
            The ID of the form to retrieve.

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFormResponseContent
            Form successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.get(
                id="id",
                hydrate=["flow_count"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, hydrate=hydrate, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            The ID of the form to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        messages: typing.Optional[FormMessagesNullable] = OMIT,
        languages: typing.Optional[FormLanguagesNullable] = OMIT,
        translations: typing.Optional[FormTranslationsNullable] = OMIT,
        nodes: typing.Optional[FormNodeListNullable] = OMIT,
        start: typing.Optional[FormStartNodeNullable] = OMIT,
        ending: typing.Optional[FormEndingNodeNullable] = OMIT,
        style: typing.Optional[FormStyleNullable] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateFormResponseContent:
        """
        Parameters
        ----------
        id : str
            The ID of the form to update.

        name : typing.Optional[str]

        messages : typing.Optional[FormMessagesNullable]

        languages : typing.Optional[FormLanguagesNullable]

        translations : typing.Optional[FormTranslationsNullable]

        nodes : typing.Optional[FormNodeListNullable]

        start : typing.Optional[FormStartNodeNullable]

        ending : typing.Optional[FormEndingNodeNullable]

        style : typing.Optional[FormStyleNullable]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateFormResponseContent
            Form successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.forms.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            messages=messages,
            languages=languages,
            translations=translations,
            nodes=nodes,
            start=start,
            ending=ending,
            style=style,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .cimd_mapped_client_fields import CimdMappedClientFields
from .cimd_validation_result import CimdValidationResult


class RegisterCimdClientResponseContent(UniversalBaseModel):
    """
    Response after successfully registering or updating a CIMD client
    """

    client_id: str = pydantic.Field()
    """
    The Auth0 client_id of the created or updated client
    """

    mapped_fields: CimdMappedClientFields
    validation: CimdValidationResult

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.get_branding_response_content import GetBrandingResponseContent
from ..types.update_branding_colors import UpdateBrandingColors
from ..types.update_branding_font import UpdateBrandingFont
from ..types.update_branding_response_content import UpdateBrandingResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBrandingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBrandingResponseContent]:
        """
        Retrieve branding settings.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBrandingResponseContent]
            Branding settings successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBrandingResponseContent,
                    parse_obj_as(
                        type_=GetBrandingResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        colors: typing.Optional[UpdateBrandingColors] = OMIT,
        favicon_url: typing.Optional[str] = OMIT,
        logo_url: typing.Optional[str] = OMIT,
        font: typing.Optional[UpdateBrandingFont] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBrandingResponseContent]:
        """
        Update branding settings.

        Parameters
        ----------
        colors : typing.Optional[UpdateBrandingColors]

        favicon_url : typing.Optional[str]
            URL for the favicon. Must use HTTPS.

        logo_url : typing.Optional[str]
            URL for the logo. Must use HTTPS.

        font : typing.Optional[UpdateBrandingFont]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBrandingResponseContent]
            Branding settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding",
            method="PATCH",
            json={
                "colors": convert_and_respect_annotation_metadata(
                    object_=colors, annotation=typing.Optional[UpdateBrandingColors], direction="write"
                ),
                "favicon_url": favicon_url,
                "logo_url": logo_url,
                "font": convert_and_respect_annotation_metadata(
                    object_=font, annotation=typing.Optional[UpdateBrandingFont], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBrandingResponseContent,
                    parse_obj_as(
                        type_=UpdateBrandingResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBrandingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBrandingResponseContent]:
        """
        Retrieve branding settings.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBrandingResponseContent]
            Branding settings successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBrandingResponseContent,
                    parse_obj_as(
                        type_=GetBrandingResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        colors: typing.Optional[UpdateBrandingColors] = OMIT,
        favicon_url: typing.Optional[str] = OMIT,
        logo_url: typing.Optional[str] = OMIT,
        font: typing.Optional[UpdateBrandingFont] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBrandingResponseContent]:
        """
        Update branding settings.

        Parameters
        ----------
        colors : typing.Optional[UpdateBrandingColors]

        favicon_url : typing.Optional[str]
            URL for the favicon. Must use HTTPS.

        logo_url : typing.Optional[str]
            URL for the logo. Must use HTTPS.

        font : typing.Optional[UpdateBrandingFont]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBrandingResponseContent]
            Branding settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding",
            method="PATCH",
            json={
                "colors": convert_and_respect_annotation_metadata(
                    object_=colors, annotation=typing.Optional[UpdateBrandingColors], direction="write"
                ),
                "favicon_url": favicon_url,
                "logo_url": logo_url,
                "font": convert_and_respect_annotation_metadata(
                    object_=font, annotation=typing.Optional[UpdateBrandingFont], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBrandingResponseContent,
                    parse_obj_as(
                        type_=UpdateBrandingResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_otp_generate_code import FlowActionOtpGenerateCode
from .flow_action_otp_verify_code import FlowActionOtpVerifyCode

FlowActionOtp = typing.Union[FlowActionOtpGenerateCode, FlowActionOtpVerifyCode]



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionFreeformScopesPaypal = ConnectionScopeArray
"""
Additional OAuth scopes to request from PayPal beyond the standard attribute scopes. Enter valid PayPal scopes from their documentation. Invalid scopes may cause authentication errors.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .cimd_mapped_client_authentication_methods_private_key_jwt import (
    CimdMappedClientAuthenticationMethodsPrivateKeyJwt,
)


class CimdMappedClientAuthenticationMethods(UniversalBaseModel):
    """
    Client authentication methods derived from the JWKS document
    """

    private_key_jwt: typing.Optional[CimdMappedClientAuthenticationMethodsPrivateKeyJwt] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.daily_stats import DailyStats
from ..types.get_active_users_count_stats_response_content import GetActiveUsersCountStatsResponseContent
from .raw_client import AsyncRawStatsClient, RawStatsClient


class StatsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawStatsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawStatsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawStatsClient
        """
        return self._raw_client

    def get_active_users_count(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActiveUsersCountStatsResponseContent:
        """
        Retrieve the number of active users that logged in during the last 30 days.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActiveUsersCountStatsResponseContent
            Number of active users successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.stats.get_active_users_count()
        """
        _response = self._raw_client.get_active_users_count(request_options=request_options)
        return _response.data

    def get_daily(
        self,
        *,
        from_: typing.Optional[str] = None,
        to: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[DailyStats]:
        """
        Retrieve the number of logins, signups and breached-password detections (subscription required) that occurred each day within a specified date range.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional first day of the date range (inclusive) in YYYYMMDD format.

        to : typing.Optional[str]
            Optional last day of the date range (inclusive) in YYYYMMDD format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DailyStats]
            Daily stats successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.stats.get_daily(
            from_="from",
            to="to",
        )
        """
        _response = self._raw_client.get_daily(from_=from_, to=to, request_options=request_options)
        return _response.data


class AsyncStatsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawStatsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawStatsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawStatsClient
        """
        return self._raw_client

    async def get_active_users_count(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActiveUsersCountStatsResponseContent:
        """
        Retrieve the number of active users that logged in during the last 30 days.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActiveUsersCountStatsResponseContent
            Number of active users successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.stats.get_active_users_count()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_active_users_count(request_options=request_options)
        return _response.data

    async def get_daily(
        self,
        *,
        from_: typing.Optional[str] = None,
        to: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[DailyStats]:
        """
        Retrieve the number of logins, signups and breached-password detections (subscription required) that occurred each day within a specified date range.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional first day of the date range (inclusive) in YYYYMMDD format.

        to : typing.Optional[str]
            Optional last day of the date range (inclusive) in YYYYMMDD format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DailyStats]
            Daily stats successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.stats.get_daily(
                from_="from",
                to="to",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_daily(from_=from_, to=to, request_options=request_options)
        return _response.data



from .conftest import get_client, verify_request_count


def test_connections_directoryProvisioning_synchronizations_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "connections.directory_provisioning.synchronizations.create.0"
    client = get_client(test_id)
    client.connections.directory_provisioning.synchronizations.create(
        id="id",
    )
    verify_request_count(test_id, "POST", "/connections/id/directory-provisioning/synchronizations", None, 1)



# This file was auto-generated by Fern from our API Definition.

ConnectionIssuer = str
"""
The identity provider's unique issuer identifier URL (e.g., https://accounts.google.com). Must match the 'iss' claim in ID tokens from the identity provider.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_ext_admin import ConnectionExtAdmin

ConnectionExtIsAdminGoogleApps = ConnectionExtAdmin
"""
Fetches the Google Directory admin flag for the signing-in user.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConnectionPasswordOptionsProfileData(UniversalBaseModel):
    """
    Personal information restriction policy to prevent use of profile data in passwords
    """

    active: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Prevents users from including profile data (like name, email) in their passwords
    """

    blocked_fields: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Blocked profile fields. An array of up to 12 entries.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.change_password_ticket_identity import ChangePasswordTicketIdentity
from ..types.change_password_ticket_response_content import ChangePasswordTicketResponseContent
from ..types.identity import Identity
from ..types.verify_email_ticket_response_content import VerifyEmailTicketResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTicketsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def verify_email(
        self,
        *,
        user_id: str,
        result_url: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VerifyEmailTicketResponseContent]:
        """
        Create an email verification ticket for a given user. An email verification ticket is a generated URL that the user can consume to verify their email address.

        Parameters
        ----------
        user_id : str
            user_id of for whom the ticket should be created.

        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using client_id or organization_id.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[Identity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VerifyEmailTicketResponseContent]
            Ticket successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tickets/email-verification",
            method="POST",
            json={
                "result_url": result_url,
                "user_id": user_id,
                "client_id": client_id,
                "organization_id": organization_id,
                "ttl_sec": ttl_sec,
                "includeEmailInRedirect": include_email_in_redirect,
                "identity": convert_and_respect_annotation_metadata(
                    object_=identity, annotation=Identity, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VerifyEmailTicketResponseContent,
                    parse_obj_as(
                        type_=VerifyEmailTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def change_password(
        self,
        *,
        result_url: typing.Optional[str] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        mark_email_as_verified: typing.Optional[bool] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[ChangePasswordTicketIdentity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ChangePasswordTicketResponseContent]:
        """
        Create a password change ticket for a given user. A password change ticket is a generated URL that the user can consume to start a reset password flow.

        Note: This endpoint does not verify the given user’s identity. If you call this endpoint within your application, you must design your application to verify the user’s identity.

        Parameters
        ----------
        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using organization_id. May be specified together with client_id when the tenant has a custom password reset page enabled and a password-reset-post-challenge Action bound.

        user_id : typing.Optional[str]
            user_id of for whom the ticket should be created.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        connection_id : typing.Optional[str]
            ID of the connection. If provided, allows the user to be specified using email instead of user_id. If you set this value, you must also send the email parameter. You cannot send user_id when specifying a connection_id.

        email : typing.Optional[str]
            Email address of the user for whom the tickets should be created. Requires the connection_id parameter. Cannot be specified when using user_id.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        mark_email_as_verified : typing.Optional[bool]
            Whether to set the email_verified attribute to true (true) or whether it should not be updated (false).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[ChangePasswordTicketIdentity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChangePasswordTicketResponseContent]
            Ticket successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tickets/password-change",
            method="POST",
            json={
                "result_url": result_url,
                "user_id": user_id,
                "client_id": client_id,
                "organization_id": organization_id,
                "connection_id": connection_id,
                "email": email,
                "ttl_sec": ttl_sec,
                "mark_email_as_verified": mark_email_as_verified,
                "includeEmailInRedirect": include_email_in_redirect,
                "identity": convert_and_respect_annotation_metadata(
                    object_=identity, annotation=ChangePasswordTicketIdentity, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangePasswordTicketResponseContent,
                    parse_obj_as(
                        type_=ChangePasswordTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTicketsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def verify_email(
        self,
        *,
        user_id: str,
        result_url: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VerifyEmailTicketResponseContent]:
        """
        Create an email verification ticket for a given user. An email verification ticket is a generated URL that the user can consume to verify their email address.

        Parameters
        ----------
        user_id : str
            user_id of for whom the ticket should be created.

        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using client_id or organization_id.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[Identity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VerifyEmailTicketResponseContent]
            Ticket successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tickets/email-verification",
            method="POST",
            json={
                "result_url": result_url,
                "user_id": user_id,
                "client_id": client_id,
                "organization_id": organization_id,
                "ttl_sec": ttl_sec,
                "includeEmailInRedirect": include_email_in_redirect,
                "identity": convert_and_respect_annotation_metadata(
                    object_=identity, annotation=Identity, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VerifyEmailTicketResponseContent,
                    parse_obj_as(
                        type_=VerifyEmailTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def change_password(
        self,
        *,
        result_url: typing.Optional[str] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        mark_email_as_verified: typing.Optional[bool] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[ChangePasswordTicketIdentity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ChangePasswordTicketResponseContent]:
        """
        Create a password change ticket for a given user. A password change ticket is a generated URL that the user can consume to start a reset password flow.

        Note: This endpoint does not verify the given user’s identity. If you call this endpoint within your application, you must design your application to verify the user’s identity.

        Parameters
        ----------
        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using organization_id. May be specified together with client_id when the tenant has a custom password reset page enabled and a password-reset-post-challenge Action bound.

        user_id : typing.Optional[str]
            user_id of for whom the ticket should be created.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        connection_id : typing.Optional[str]
            ID of the connection. If provided, allows the user to be specified using email instead of user_id. If you set this value, you must also send the email parameter. You cannot send user_id when specifying a connection_id.

        email : typing.Optional[str]
            Email address of the user for whom the tickets should be created. Requires the connection_id parameter. Cannot be specified when using user_id.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        mark_email_as_verified : typing.Optional[bool]
            Whether to set the email_verified attribute to true (true) or whether it should not be updated (false).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[ChangePasswordTicketIdentity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChangePasswordTicketResponseContent]
            Ticket successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tickets/password-change",
            method="POST",
            json={
                "result_url": result_url,
                "user_id": user_id,
                "client_id": client_id,
                "organization_id": organization_id,
                "connection_id": connection_id,
                "email": email,
                "ttl_sec": ttl_sec,
                "mark_email_as_verified": mark_email_as_verified,
                "includeEmailInRedirect": include_email_in_redirect,
                "identity": convert_and_respect_annotation_metadata(
                    object_=identity, annotation=ChangePasswordTicketIdentity, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangePasswordTicketResponseContent,
                    parse_obj_as(
                        type_=ChangePasswordTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionScopeAmazon = ConnectionScopeArray
"""
OAuth 2.0 scopes that will be requested from Amazon during authorization. This is automatically populated based on the profile and postal_code settings, plus any freeform_scopes.
"""



from .conftest import get_client, verify_request_count


def test_deviceCredentials_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "device_credentials.list_.0"
    client = get_client(test_id)
    client.device_credentials.list(
        page=1,
        per_page=1,
        include_totals=True,
        fields="fields",
        include_fields=True,
        user_id="user_id",
        client_id="client_id",
        type="public_key",
    )
    verify_request_count(
        test_id,
        "GET",
        "/device-credentials",
        {
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
            "fields": "fields",
            "include_fields": "true",
            "user_id": "user_id",
            "client_id": "client_id",
            "type": "public_key",
        },
        1,
    )


def test_deviceCredentials_create_public_key() -> None:
    """Test createPublicKey endpoint with WireMock"""
    test_id = "device_credentials.create_public_key.0"
    client = get_client(test_id)
    client.device_credentials.create_public_key(
        device_name="device_name",
        type="public_key",
        value="value",
        device_id="device_id",
    )
    verify_request_count(test_id, "POST", "/device-credentials", None, 1)


def test_deviceCredentials_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "device_credentials.delete.0"
    client = get_client(test_id)
    client.device_credentials.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/device-credentials/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.conflict_error import ConflictError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.create_self_service_profile_sso_ticket_response_content import (
    CreateSelfServiceProfileSsoTicketResponseContent,
)
from ...types.self_service_profile_sso_ticket_connection_config import SelfServiceProfileSsoTicketConnectionConfig
from ...types.self_service_profile_sso_ticket_domain_aliases_config import (
    SelfServiceProfileSsoTicketDomainAliasesConfig,
)
from ...types.self_service_profile_sso_ticket_enabled_features import SelfServiceProfileSsoTicketEnabledFeatures
from ...types.self_service_profile_sso_ticket_enabled_organization import SelfServiceProfileSsoTicketEnabledOrganization
from ...types.self_service_profile_sso_ticket_provisioning_config import SelfServiceProfileSsoTicketProvisioningConfig
from ...types.third_party_client_access_config import ThirdPartyClientAccessConfig
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSsoTicketClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        id: str,
        *,
        connection_id: typing.Optional[str] = OMIT,
        connection_config: typing.Optional[SelfServiceProfileSsoTicketConnectionConfig] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        enabled_organizations: typing.Optional[typing.Sequence[SelfServiceProfileSsoTicketEnabledOrganization]] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        domain_aliases_config: typing.Optional[SelfServiceProfileSsoTicketDomainAliasesConfig] = OMIT,
        provisioning_config: typing.Optional[SelfServiceProfileSsoTicketProvisioningConfig] = OMIT,
        use_for_organization_discovery: typing.Optional[bool] = OMIT,
        third_party_client_access_config: typing.Optional[ThirdPartyClientAccessConfig] = OMIT,
        enabled_features: typing.Optional[SelfServiceProfileSsoTicketEnabledFeatures] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSelfServiceProfileSsoTicketResponseContent]:
        """
        Creates an access ticket to initiate the Self-Service Enterprise Configuration flow using a self-service profile.

        Parameters
        ----------
        id : str
            The id of the self-service profile to retrieve

        connection_id : typing.Optional[str]
            If provided, this will allow editing of the provided connection during the Self-Service Enterprise Configuration flow

        connection_config : typing.Optional[SelfServiceProfileSsoTicketConnectionConfig]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            List of client_ids that the connection will be enabled for.

        enabled_organizations : typing.Optional[typing.Sequence[SelfServiceProfileSsoTicketEnabledOrganization]]
            List of organizations that the connection will be enabled for.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        domain_aliases_config : typing.Optional[SelfServiceProfileSsoTicketDomainAliasesConfig]

        provisioning_config : typing.Optional[SelfServiceProfileSsoTicketProvisioningConfig]

        use_for_organization_discovery : typing.Optional[bool]
            Indicates whether a verified domain should be used for organization discovery during authentication.

        third_party_client_access_config : typing.Optional[ThirdPartyClientAccessConfig]

        enabled_features : typing.Optional[SelfServiceProfileSsoTicketEnabledFeatures]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSelfServiceProfileSsoTicketResponseContent]
            Self-Service Enterprise Configuration Access Ticket successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}/sso-ticket",
            method="POST",
            json={
                "connection_id": connection_id,
                "connection_config": convert_and_respect_annotation_metadata(
                    object_=connection_config, annotation=SelfServiceProfileSsoTicketConnectionConfig, direction="write"
                ),
                "enabled_clients": enabled_clients,
                "enabled_organizations": convert_and_respect_annotation_metadata(
                    object_=enabled_organizations,
                    annotation=typing.Sequence[SelfServiceProfileSsoTicketEnabledOrganization],
                    direction="write",
                ),
                "ttl_sec": ttl_sec,
                "domain_aliases_config": convert_and_respect_annotation_metadata(
                    object_=domain_aliases_config,
                    annotation=SelfServiceProfileSsoTicketDomainAliasesConfig,
                    direction="write",
                ),
                "provisioning_config": convert_and_respect_annotation_metadata(
                    object_=provisioning_config,
                    annotation=SelfServiceProfileSsoTicketProvisioningConfig,
                    direction="write",
                ),
                "use_for_organization_discovery": use_for_organization_discovery,
                "third_party_client_access_config": convert_and_respect_annotation_metadata(
                    object_=third_party_client_access_config, annotation=ThirdPartyClientAccessConfig, direction="write"
                ),
                "enabled_features": convert_and_respect_annotation_metadata(
                    object_=enabled_features, annotation=SelfServiceProfileSsoTicketEnabledFeatures, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSelfServiceProfileSsoTicketResponseContent,
                    parse_obj_as(
                        type_=CreateSelfServiceProfileSsoTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def revoke(
        self, profile_id: str, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Revokes a Self-Service Enterprise Configuration access ticket and invalidates associated sessions. The ticket will no longer be accepted to initiate a Self-Service Enterprise Configuration session. If any users have already started a session through this ticket, their session will be terminated. Clients should expect a `202 Accepted` response upon successful processing, indicating that the request has been acknowledged and that the revocation is underway but may not be fully completed at the time of response. If the specified ticket does not exist, a `202 Accepted` response is also returned, signaling that no further action is required.
        Clients should treat these `202` responses as an acknowledgment that the request has been accepted and is in progress, even if the ticket was not found.

        Parameters
        ----------
        profile_id : str
            The id of the self-service profile

        id : str
            The id of the ticket to revoke

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(profile_id)}/sso-ticket/{encode_path_param(id)}/revoke",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSsoTicketClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        id: str,
        *,
        connection_id: typing.Optional[str] = OMIT,
        connection_config: typing.Optional[SelfServiceProfileSsoTicketConnectionConfig] = OMIT,
        enabled_clients: typing.Optional[typing.Sequence[str]] = OMIT,
        enabled_organizations: typing.Optional[typing.Sequence[SelfServiceProfileSsoTicketEnabledOrganization]] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        domain_aliases_config: typing.Optional[SelfServiceProfileSsoTicketDomainAliasesConfig] = OMIT,
        provisioning_config: typing.Optional[SelfServiceProfileSsoTicketProvisioningConfig] = OMIT,
        use_for_organization_discovery: typing.Optional[bool] = OMIT,
        third_party_client_access_config: typing.Optional[ThirdPartyClientAccessConfig] = OMIT,
        enabled_features: typing.Optional[SelfServiceProfileSsoTicketEnabledFeatures] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSelfServiceProfileSsoTicketResponseContent]:
        """
        Creates an access ticket to initiate the Self-Service Enterprise Configuration flow using a self-service profile.

        Parameters
        ----------
        id : str
            The id of the self-service profile to retrieve

        connection_id : typing.Optional[str]
            If provided, this will allow editing of the provided connection during the Self-Service Enterprise Configuration flow

        connection_config : typing.Optional[SelfServiceProfileSsoTicketConnectionConfig]

        enabled_clients : typing.Optional[typing.Sequence[str]]
            List of client_ids that the connection will be enabled for.

        enabled_organizations : typing.Optional[typing.Sequence[SelfServiceProfileSsoTicketEnabledOrganization]]
            List of organizations that the connection will be enabled for.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        domain_aliases_config : typing.Optional[SelfServiceProfileSsoTicketDomainAliasesConfig]

        provisioning_config : typing.Optional[SelfServiceProfileSsoTicketProvisioningConfig]

        use_for_organization_discovery : typing.Optional[bool]
            Indicates whether a verified domain should be used for organization discovery during authentication.

        third_party_client_access_config : typing.Optional[ThirdPartyClientAccessConfig]

        enabled_features : typing.Optional[SelfServiceProfileSsoTicketEnabledFeatures]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSelfServiceProfileSsoTicketResponseContent]
            Self-Service Enterprise Configuration Access Ticket successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}/sso-ticket",
            method="POST",
            json={
                "connection_id": connection_id,
                "connection_config": convert_and_respect_annotation_metadata(
                    object_=connection_config, annotation=SelfServiceProfileSsoTicketConnectionConfig, direction="write"
                ),
                "enabled_clients": enabled_clients,
                "enabled_organizations": convert_and_respect_annotation_metadata(
                    object_=enabled_organizations,
                    annotation=typing.Sequence[SelfServiceProfileSsoTicketEnabledOrganization],
                    direction="write",
                ),
                "ttl_sec": ttl_sec,
                "domain_aliases_config": convert_and_respect_annotation_metadata(
                    object_=domain_aliases_config,
                    annotation=SelfServiceProfileSsoTicketDomainAliasesConfig,
                    direction="write",
                ),
                "provisioning_config": convert_and_respect_annotation_metadata(
                    object_=provisioning_config,
                    annotation=SelfServiceProfileSsoTicketProvisioningConfig,
                    direction="write",
                ),
                "use_for_organization_discovery": use_for_organization_discovery,
                "third_party_client_access_config": convert_and_respect_annotation_metadata(
                    object_=third_party_client_access_config, annotation=ThirdPartyClientAccessConfig, direction="write"
                ),
                "enabled_features": convert_and_respect_annotation_metadata(
                    object_=enabled_features, annotation=SelfServiceProfileSsoTicketEnabledFeatures, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSelfServiceProfileSsoTicketResponseContent,
                    parse_obj_as(
                        type_=CreateSelfServiceProfileSsoTicketResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def revoke(
        self, profile_id: str, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Revokes a Self-Service Enterprise Configuration access ticket and invalidates associated sessions. The ticket will no longer be accepted to initiate a Self-Service Enterprise Configuration session. If any users have already started a session through this ticket, their session will be terminated. Clients should expect a `202 Accepted` response upon successful processing, indicating that the request has been acknowledged and that the revocation is underway but may not be fully completed at the time of response. If the specified ticket does not exist, a `202 Accepted` response is also returned, signaling that no further action is required.
        Clients should treat these `202` responses as an acknowledgment that the request has been accepted and is in progress, even if the ticket was not found.

        Parameters
        ----------
        profile_id : str
            The id of the self-service profile

        id : str
            The id of the ticket to revoke

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(profile_id)}/sso-ticket/{encode_path_param(id)}/revoke",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.list_role_users_paginated_response_content import ListRoleUsersPaginatedResponseContent
from ...types.role_user import RoleUser
from .raw_client import AsyncRawUsersClient, RawUsersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[RoleUser, ListRoleUsersPaginatedResponseContent]:
        """
        Retrieve list of users associated with a specific role. For Dashboard instructions, review [View Users Assigned to Roles](https://auth0.com/docs/manage-users/access-control/configure-core-rbac/roles/view-users-assigned-to-roles).

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 organization members.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        id : str
            ID of the role to retrieve a list of users associated with.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RoleUser, ListRoleUsersPaginatedResponseContent]
            Role users successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.roles.users.list(
            id="id",
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(id, from_=from_, take=take, request_options=request_options)

    def assign(
        self, id: str, *, users: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Assign one or more users to an existing user role. To learn more, review [Role-Based Access Control](https://auth0.com/docs/manage-users/access-control/rbac).

        **Note**: New roles cannot be created through this action.

        Parameters
        ----------
        id : str
            ID of the role to assign users to.

        users : typing.Sequence[str]
            user_id's of the users to assign the role to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.roles.users.assign(
            id="id",
            users=["users"],
        )
        """
        _response = self._raw_client.assign(id, users=users, request_options=request_options)
        return _response.data


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[RoleUser, ListRoleUsersPaginatedResponseContent]:
        """
        Retrieve list of users associated with a specific role. For Dashboard instructions, review [View Users Assigned to Roles](https://auth0.com/docs/manage-users/access-control/configure-core-rbac/roles/view-users-assigned-to-roles).

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 organization members.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        id : str
            ID of the role to retrieve a list of users associated with.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RoleUser, ListRoleUsersPaginatedResponseContent]
            Role users successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.roles.users.list(
                id="id",
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(id, from_=from_, take=take, request_options=request_options)

    async def assign(
        self, id: str, *, users: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Assign one or more users to an existing user role. To learn more, review [Role-Based Access Control](https://auth0.com/docs/manage-users/access-control/rbac).

        **Note**: New roles cannot be created through this action.

        Parameters
        ----------
        id : str
            ID of the role to assign users to.

        users : typing.Sequence[str]
            user_id's of the users to assign the role to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.roles.users.assign(
                id="id",
                users=["users"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.assign(id, users=users, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionDeletedObject0OptionsTypeEnum = typing.Union[
    typing.Literal["back_channel", "front_channel"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

CspDirectives = typing.Dict[str, typing.List[str]]
"""
CSP directives map. Keys are directive names, values are arrays of directive values.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionsMetadata = typing.Dict[str, typing.Optional[str]]
"""
Metadata associated with the connection in the form of an object with string values (max 255 chars).  Maximum of 10 metadata properties allowed.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .resource_server_token_encryption_algorithm_enum import ResourceServerTokenEncryptionAlgorithmEnum


class ResourceServerTokenEncryptionKey(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the encryption key.
    """

    alg: ResourceServerTokenEncryptionAlgorithmEnum
    kid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Key ID.
    """

    pem: str = pydantic.Field()
    """
    PEM-formatted public key. Must be JSON escaped.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_clearbit_api_key import CreateFlowsVaultConnectionClearbitApiKey
from .create_flows_vault_connection_clearbit_uninitialized import CreateFlowsVaultConnectionClearbitUninitialized

CreateFlowsVaultConnectionClearbit = typing.Union[
    CreateFlowsVaultConnectionClearbitApiKey, CreateFlowsVaultConnectionClearbitUninitialized
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.pagination import AsyncPager, SyncPager
from ....core.request_options import RequestOptions
from ....types.create_verifiable_credential_template_response_content import (
    CreateVerifiableCredentialTemplateResponseContent,
)
from ....types.get_verifiable_credential_template_response_content import GetVerifiableCredentialTemplateResponseContent
from ....types.list_verifiable_credential_templates_paginated_response_content import (
    ListVerifiableCredentialTemplatesPaginatedResponseContent,
)
from ....types.mdl_presentation_request import MdlPresentationRequest
from ....types.update_verifiable_credential_template_response_content import (
    UpdateVerifiableCredentialTemplateResponseContent,
)
from ....types.verifiable_credential_template_response import VerifiableCredentialTemplateResponse
from .raw_client import AsyncRawTemplatesClient, RawTemplatesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTemplatesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]:
        """
        List verifiable credential templates.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]
            Templates successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.verifiable_credentials.verification.templates.list(
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(from_=from_, take=take, request_options=request_options)

    def create(
        self,
        *,
        name: str,
        type: str,
        dialect: str,
        presentation: MdlPresentationRequest,
        well_known_trusted_issuers: str,
        custom_certificate_authority: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateVerifiableCredentialTemplateResponseContent:
        """
        Create a verifiable credential template.

        Parameters
        ----------
        name : str

        type : str

        dialect : str

        presentation : MdlPresentationRequest

        well_known_trusted_issuers : str

        custom_certificate_authority : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateVerifiableCredentialTemplateResponseContent
            Template successfully created.

        Examples
        --------
        from auth0 import (
            Auth0,
            MdlPresentationProperties,
            MdlPresentationRequest,
            MdlPresentationRequestProperties,
        )

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.verifiable_credentials.verification.templates.create(
            name="name",
            type="type",
            dialect="dialect",
            presentation=MdlPresentationRequest(
                org_iso_18013_5_1_m_dl=MdlPresentationRequestProperties(
                    org_iso_18013_5_1=MdlPresentationProperties(),
                ),
            ),
            well_known_trusted_issuers="well_known_trusted_issuers",
        )
        """
        _response = self._raw_client.create(
            name=name,
            type=type,
            dialect=dialect,
            presentation=presentation,
            well_known_trusted_issuers=well_known_trusted_issuers,
            custom_certificate_authority=custom_certificate_authority,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetVerifiableCredentialTemplateResponseContent:
        """
        Get a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVerifiableCredentialTemplateResponseContent
            Template successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.verifiable_credentials.verification.templates.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.verifiable_credentials.verification.templates.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        type: typing.Optional[str] = OMIT,
        dialect: typing.Optional[str] = OMIT,
        presentation: typing.Optional[MdlPresentationRequest] = OMIT,
        well_known_trusted_issuers: typing.Optional[str] = OMIT,
        version: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateVerifiableCredentialTemplateResponseContent:
        """
        Update a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        name : typing.Optional[str]

        type : typing.Optional[str]

        dialect : typing.Optional[str]

        presentation : typing.Optional[MdlPresentationRequest]

        well_known_trusted_issuers : typing.Optional[str]

        version : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateVerifiableCredentialTemplateResponseContent
            Template successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.verifiable_credentials.verification.templates.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            type=type,
            dialect=dialect,
            presentation=presentation,
            well_known_trusted_issuers=well_known_trusted_issuers,
            version=version,
            request_options=request_options,
        )
        return _response.data


class AsyncTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTemplatesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]:
        """
        List verifiable credential templates.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[VerifiableCredentialTemplateResponse, ListVerifiableCredentialTemplatesPaginatedResponseContent]
            Templates successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.verifiable_credentials.verification.templates.list(
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(from_=from_, take=take, request_options=request_options)

    async def create(
        self,
        *,
        name: str,
        type: str,
        dialect: str,
        presentation: MdlPresentationRequest,
        well_known_trusted_issuers: str,
        custom_certificate_authority: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateVerifiableCredentialTemplateResponseContent:
        """
        Create a verifiable credential template.

        Parameters
        ----------
        name : str

        type : str

        dialect : str

        presentation : MdlPresentationRequest

        well_known_trusted_issuers : str

        custom_certificate_authority : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateVerifiableCredentialTemplateResponseContent
            Template successfully created.

        Examples
        --------
        import asyncio

        from auth0 import (
            AsyncAuth0,
            MdlPresentationProperties,
            MdlPresentationRequest,
            MdlPresentationRequestProperties,
        )

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.verifiable_credentials.verification.templates.create(
                name="name",
                type="type",
                dialect="dialect",
                presentation=MdlPresentationRequest(
                    org_iso_18013_5_1_m_dl=MdlPresentationRequestProperties(
                        org_iso_18013_5_1=MdlPresentationProperties(),
                    ),
                ),
                well_known_trusted_issuers="well_known_trusted_issuers",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            type=type,
            dialect=dialect,
            presentation=presentation,
            well_known_trusted_issuers=well_known_trusted_issuers,
            custom_certificate_authority=custom_certificate_authority,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetVerifiableCredentialTemplateResponseContent:
        """
        Get a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVerifiableCredentialTemplateResponseContent
            Template successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.verifiable_credentials.verification.templates.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.verifiable_credentials.verification.templates.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        type: typing.Optional[str] = OMIT,
        dialect: typing.Optional[str] = OMIT,
        presentation: typing.Optional[MdlPresentationRequest] = OMIT,
        well_known_trusted_issuers: typing.Optional[str] = OMIT,
        version: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateVerifiableCredentialTemplateResponseContent:
        """
        Update a verifiable credential template.

        Parameters
        ----------
        id : str
            ID of the template to retrieve.

        name : typing.Optional[str]

        type : typing.Optional[str]

        dialect : typing.Optional[str]

        presentation : typing.Optional[MdlPresentationRequest]

        well_known_trusted_issuers : typing.Optional[str]

        version : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateVerifiableCredentialTemplateResponseContent
            Template successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.verifiable_credentials.verification.templates.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            type=type,
            dialect=dialect,
            presentation=presentation,
            well_known_trusted_issuers=well_known_trusted_issuers,
            version=version,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .resource_server_proof_of_possession_mechanism_enum import ResourceServerProofOfPossessionMechanismEnum
from .resource_server_proof_of_possession_required_for_enum import ResourceServerProofOfPossessionRequiredForEnum


class ResourceServerProofOfPossession(UniversalBaseModel):
    """
    Proof-of-Possession configuration for access tokens
    """

    mechanism: ResourceServerProofOfPossessionMechanismEnum
    required: bool = pydantic.Field()
    """
    Whether the use of Proof-of-Possession is required for the resource server
    """

    required_for: typing.Optional[ResourceServerProofOfPossessionRequiredForEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.change_password_ticket_identity import ChangePasswordTicketIdentity
from ..types.change_password_ticket_response_content import ChangePasswordTicketResponseContent
from ..types.identity import Identity
from ..types.verify_email_ticket_response_content import VerifyEmailTicketResponseContent
from .raw_client import AsyncRawTicketsClient, RawTicketsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TicketsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTicketsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTicketsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTicketsClient
        """
        return self._raw_client

    def verify_email(
        self,
        *,
        user_id: str,
        result_url: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VerifyEmailTicketResponseContent:
        """
        Create an email verification ticket for a given user. An email verification ticket is a generated URL that the user can consume to verify their email address.

        Parameters
        ----------
        user_id : str
            user_id of for whom the ticket should be created.

        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using client_id or organization_id.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[Identity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VerifyEmailTicketResponseContent
            Ticket successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.tickets.verify_email(
            user_id="user_id",
        )
        """
        _response = self._raw_client.verify_email(
            user_id=user_id,
            result_url=result_url,
            client_id=client_id,
            organization_id=organization_id,
            ttl_sec=ttl_sec,
            include_email_in_redirect=include_email_in_redirect,
            identity=identity,
            request_options=request_options,
        )
        return _response.data

    def change_password(
        self,
        *,
        result_url: typing.Optional[str] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        mark_email_as_verified: typing.Optional[bool] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[ChangePasswordTicketIdentity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChangePasswordTicketResponseContent:
        """
        Create a password change ticket for a given user. A password change ticket is a generated URL that the user can consume to start a reset password flow.

        Note: This endpoint does not verify the given user’s identity. If you call this endpoint within your application, you must design your application to verify the user’s identity.

        Parameters
        ----------
        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using organization_id. May be specified together with client_id when the tenant has a custom password reset page enabled and a password-reset-post-challenge Action bound.

        user_id : typing.Optional[str]
            user_id of for whom the ticket should be created.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        connection_id : typing.Optional[str]
            ID of the connection. If provided, allows the user to be specified using email instead of user_id. If you set this value, you must also send the email parameter. You cannot send user_id when specifying a connection_id.

        email : typing.Optional[str]
            Email address of the user for whom the tickets should be created. Requires the connection_id parameter. Cannot be specified when using user_id.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        mark_email_as_verified : typing.Optional[bool]
            Whether to set the email_verified attribute to true (true) or whether it should not be updated (false).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[ChangePasswordTicketIdentity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChangePasswordTicketResponseContent
            Ticket successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.tickets.change_password()
        """
        _response = self._raw_client.change_password(
            result_url=result_url,
            user_id=user_id,
            client_id=client_id,
            organization_id=organization_id,
            connection_id=connection_id,
            email=email,
            ttl_sec=ttl_sec,
            mark_email_as_verified=mark_email_as_verified,
            include_email_in_redirect=include_email_in_redirect,
            identity=identity,
            request_options=request_options,
        )
        return _response.data


class AsyncTicketsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTicketsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTicketsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTicketsClient
        """
        return self._raw_client

    async def verify_email(
        self,
        *,
        user_id: str,
        result_url: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VerifyEmailTicketResponseContent:
        """
        Create an email verification ticket for a given user. An email verification ticket is a generated URL that the user can consume to verify their email address.

        Parameters
        ----------
        user_id : str
            user_id of for whom the ticket should be created.

        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using client_id or organization_id.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[Identity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VerifyEmailTicketResponseContent
            Ticket successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tickets.verify_email(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.verify_email(
            user_id=user_id,
            result_url=result_url,
            client_id=client_id,
            organization_id=organization_id,
            ttl_sec=ttl_sec,
            include_email_in_redirect=include_email_in_redirect,
            identity=identity,
            request_options=request_options,
        )
        return _response.data

    async def change_password(
        self,
        *,
        result_url: typing.Optional[str] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        ttl_sec: typing.Optional[int] = OMIT,
        mark_email_as_verified: typing.Optional[bool] = OMIT,
        include_email_in_redirect: typing.Optional[bool] = OMIT,
        identity: typing.Optional[ChangePasswordTicketIdentity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChangePasswordTicketResponseContent:
        """
        Create a password change ticket for a given user. A password change ticket is a generated URL that the user can consume to start a reset password flow.

        Note: This endpoint does not verify the given user’s identity. If you call this endpoint within your application, you must design your application to verify the user’s identity.

        Parameters
        ----------
        result_url : typing.Optional[str]
            URL the user will be redirected to in the classic Universal Login experience once the ticket is used. Cannot be specified when using organization_id. May be specified together with client_id when the tenant has a custom password reset page enabled and a password-reset-post-challenge Action bound.

        user_id : typing.Optional[str]
            user_id of for whom the ticket should be created.

        client_id : typing.Optional[str]
            ID of the client (application). If provided for tenants using the New Universal Login experience, the email template and UI displays application details, and the user is prompted to redirect to the application's <a target='' href='https://auth0.com/docs/authenticate/login/auth0-universal-login/configure-default-login-routes#completing-the-password-reset-flow'>default login route</a> after the ticket is used. client_id is required to use the <a target='' href='https://auth0.com/docs/customize/actions/flows-and-triggers/post-change-password-flow'>Password Reset Post Challenge</a> trigger.

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        connection_id : typing.Optional[str]
            ID of the connection. If provided, allows the user to be specified using email instead of user_id. If you set this value, you must also send the email parameter. You cannot send user_id when specifying a connection_id.

        email : typing.Optional[str]
            Email address of the user for whom the tickets should be created. Requires the connection_id parameter. Cannot be specified when using user_id.

        ttl_sec : typing.Optional[int]
            Number of seconds for which the ticket is valid before expiration. If unspecified or set to 0, this value defaults to 432000 seconds (5 days).

        mark_email_as_verified : typing.Optional[bool]
            Whether to set the email_verified attribute to true (true) or whether it should not be updated (false).

        include_email_in_redirect : typing.Optional[bool]
            Whether to include the email address as part of the returnUrl in the reset_email (true), or not (false).

        identity : typing.Optional[ChangePasswordTicketIdentity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChangePasswordTicketResponseContent
            Ticket successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tickets.change_password()


        asyncio.run(main())
        """
        _response = await self._raw_client.change_password(
            result_url=result_url,
            user_id=user_id,
            client_id=client_id,
            organization_id=organization_id,
            connection_id=connection_id,
            email=email,
            ttl_sec=ttl_sec,
            mark_email_as_verified=mark_email_as_verified,
            include_email_in_redirect=include_email_in_redirect,
            identity=identity,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .scim_mapping_item import ScimMappingItem


class ScimConfiguration(UniversalBaseModel):
    connection_id: str = pydantic.Field()
    """
    The connection's identifier
    """

    connection_name: str = pydantic.Field()
    """
    The connection's name
    """

    strategy: str = pydantic.Field()
    """
    The connection's strategy
    """

    tenant_name: str = pydantic.Field()
    """
    The tenant's name
    """

    user_id_attribute: str = pydantic.Field()
    """
    User ID attribute for generating unique user ids
    """

    mapping: typing.List[ScimMappingItem] = pydantic.Field()
    """
    The mapping between auth0 and SCIM
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The ISO 8601 date and time the SCIM configuration was created at
    """

    updated_on: dt.datetime = pydantic.Field()
    """
    The ISO 8601 date and time the SCIM configuration was last updated on
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_base import ActionBase
from .action_error import ActionError
from .action_module_reference import ActionModuleReference
from .action_secret_response import ActionSecretResponse
from .action_trigger import ActionTrigger
from .action_version_build_status_enum import ActionVersionBuildStatusEnum
from .action_version_dependency import ActionVersionDependency


class GetActionVersionResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique id of an action version.
    """

    action_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the action to which this version belongs.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The source code of this specific version of the action.
    """

    dependencies: typing.Optional[typing.List[ActionVersionDependency]] = pydantic.Field(default=None)
    """
    The list of third party npm modules, and their versions, that this specific version depends on. 
    """

    deployed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if this specific version is the currently one deployed.
    """

    runtime: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Node runtime. For example: `node22`
    """

    secrets: typing.Optional[typing.List[ActionSecretResponse]] = pydantic.Field(default=None)
    """
    The list of secrets that are included in an action or a version of an action.
    """

    status: typing.Optional[ActionVersionBuildStatusEnum] = None
    number: typing.Optional[float] = pydantic.Field(default=None)
    """
    The index of this version in list of versions for the action.
    """

    errors: typing.Optional[typing.List[ActionError]] = pydantic.Field(default=None)
    """
    Any errors that occurred while the version was being built.
    """

    action: typing.Optional[ActionBase] = None
    built_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this version was built successfully.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this version was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when a version was updated. Versions are never updated externally. Only Auth0 will update an action version as it is being built.
    """

    supported_triggers: typing.Optional[typing.List[ActionTrigger]] = pydantic.Field(default=None)
    """
    The list of triggers that this version supports. At this time, a version can only target a single trigger at a time.
    """

    modules: typing.Optional[typing.List[ActionModuleReference]] = pydantic.Field(default=None)
    """
    The list of action modules and their versions used by this action version.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .patch_rate_limit_policy_configuration_request_content_action_action import (
    PatchRateLimitPolicyConfigurationRequestContentActionAction,
)


class PatchRateLimitPolicyConfigurationRequestContentAction(UniversalBaseModel):
    action: PatchRateLimitPolicyConfigurationRequestContentActionAction = pydantic.Field()
    """
    Determines the action to take when the rate limit is exceeded.
    """

    limit: int = pydantic.Field()
    """
    The maximum number of requests allowed per second.
    """

    redirect_uri: str = pydantic.Field()
    """
    The HTTPS URI to redirect to when the rate limit is exceeded.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionUserinfoEncryptionAlgValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWE [JWE] encryption algorithms (alg values) [JWA] supported by the UserInfo Endpoint to encode the Claims in a JWT [JWT].
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class GetJobSummary(UniversalBaseModel):
    """
    Job execution summary.
    """

    failed: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of failed operations.
    """

    updated: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of updated records.
    """

    inserted: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of inserted records.
    """

    total: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of operations.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_org_group_role_deleted_object_group_0 import (
    EventStreamCloudEventOrgGroupRoleDeletedObjectGroup0,
)
from .event_stream_cloud_event_org_group_role_deleted_object_group_1 import (
    EventStreamCloudEventOrgGroupRoleDeletedObjectGroup1,
)
from .event_stream_cloud_event_org_group_role_deleted_object_group_2 import (
    EventStreamCloudEventOrgGroupRoleDeletedObjectGroup2,
)

EventStreamCloudEventOrgGroupRoleDeletedObjectGroup = typing.Union[
    EventStreamCloudEventOrgGroupRoleDeletedObjectGroup0,
    EventStreamCloudEventOrgGroupRoleDeletedObjectGroup1,
    EventStreamCloudEventOrgGroupRoleDeletedObjectGroup2,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .update_enabled_client_connections_request_content_item import UpdateEnabledClientConnectionsRequestContentItem

UpdateEnabledClientConnectionsRequestContent = typing.List[UpdateEnabledClientConnectionsRequestContentItem]
